from setux.core import __version__
from setux.logger import logger, info
from setux.core.module import Module


class Distro(Module):
    '''Show target infos
    '''
    def deploy(self, target, **kw):
        user = target.login.name
        kernel = target.kernel
        python = target.run('python3 -V')[1][0]
        with logger.quiet():
            try:
                addr = target.net.addr
            except:
                addr = '!'

        infos =  f'''
        target : {target}
        distro : {target.distro.name}
        python : {python}
        os     : {kernel.name} {kernel.version} / {kernel.arch}
        user   : {user}
        host   : {target.system.hostname} : {addr}
        setux  : {__version__}
        '''

        info(infos)

        return True



from setux.core.module import Module


class Distro(Module):
    '''Check Target Reachability
    '''

    register = 'ping'

    def deploy(self, target, **kw):
        pong = kw.get('pong', 'pong')
        ret, out, err = target.run('echo', pong, report='verbose')
        response = out[0]
        return response==pong


class Debian(Distro):
    def deploy(self, target, **kw):
        return super().deploy(target, pong='Debian')


class FreeBSD(Distro):
    def deploy(self, target, **kw):
        return super().deploy(target, pong='FreeBSD')



from setux.core.module import Module
from setux.logger import error


class Distro(Module):
    '''Download File

    kw:
        url : File's URL
        dst : File's Dest (defaults to "downloaded"
    '''

    register = 'download'

    @property
    def label(self):
        return 'Download'

    def deploy(self, target, **kw):
        url, dst = kw['url'], kw.get('dst')
        dst = dst or 'downloaded'
        try:
            ret, out, err = target.run(f'wget2 -q {url} -O {dst}')
            if ret!=0:
                raise RuntimeError
        except:
            try:
                # curl >= 7.75
                ret, out, err = target.run(f'curl -sfL -w "%{{errormsg}}\n" {url} -o {dst}')
                if ret!=0:
                    raise RuntimeError
            except:
                try:
                    ret, out, err = target.run(f'http --download {url} -o {dst}')
                    if ret!=0:
                        raise RuntimeError
                except:
                    target.Package.install('wget')
                    ret, out, err = target.run(f'wget -q {url} -O {dst}')
                    if ret!=0:
                        msg = err[0]
                        error(msg)
                        raise RuntimeError(msg)
        # target(f'ls -l {dst}')
        return True




from setux.core.module import Module


class Distro(Module):
    '''Minimum System Requieremnts
    '''
    def do_deploy(self, target, **kw):
        return self.install(target,
            pkg = 'tmux vim',
        )


class FreeBSD_13(Distro):
    def do_deploy(self, target, **kw):
        return self.install(target,
            pkg = 'sudo bash python rust',
        )


class Fedora(Distro):
    def do_deploy(self, target, **kw):
        ok = self.install(target,
            pkg = '''
                langpacks-fr
                langpacks-en
                glibc-all-langpacks
            ''',
        )
        if ok:
            ret, out, err = self.run('localectl set-locale LANG=fr_FR.UTF-8')
            ok = ret == 0
        return ok



from setux.core.module import Module


class Distro(Module):
    '''Send Public Key to Target
    kw:
        usr : User name
        pub : Public key
    '''

    def deploy(self, target, **kw):

        usr = kw['usr']
        pub = kw['pub']

        user = target.user(usr)

        path = f'/home/{usr}/.ssh'
        name = f'authorized_keys'
        full = f'{path}/{name}'

        ok = target.dir(
            path, mode='700', user=usr, group=user.group.name
        )

        if not ok:
            return False

        target.send(pub, full)

        ok = target.file(
            full, mode='600', user=usr, group=user.group.name
        )

        return ok



from setux.core.module import Module


class Distro(Module):
    '''Setux self test

    - test deploy
    - test remote
    - test run
    - test REPL
    '''

    def deploy(self, target, **kw):

        target.deploy('infos')
        target.pip.install('setux_plus')
        target.remote('infos')
        target.pip.install('setux_repl')
        target('setux infos')

        return True



from re import compile

from setux.core.module import Module
from setux.logger import debug


class Distro(Module):
    '''Update Config File (sed)
    '''
    def deploy(self, target, *, cfg, src=None, dst):
        cont = target.read(cfg, report='quiet')
        if dst in cont: return True
        msg = [f'{cfg} <>']
        if src:
            sre = compile(src).search
            updated, found = [], False
            for line in cont.split('\n'):
                if sre(line):
                    found = True
                    msg.append(f' <  {line}')
                    msg.append(f'  > {dst}')
                    updated.append(dst)
                else:
                    updated.append(line)
            if not found:
                updated.append(dst)
            target.write(cfg, '\n'.join(updated)+'\n', report='quiet')
        else:
            msg.append(f'  > {dst}')
            target.write(cfg, f'{cont}{dst}\n', report='quiet')

        debug('\n'.join(msg))
        return True



from setux.core.module import Module


# pylint: disable=arguments-differ


class Distro(Module):
    '''Add User to sudoers

    arg:
        user : user name
    '''

    def deploy(self, target, *, user, d=True):

        grp = target.groups.fetch(user)
        grp.add('wheel')

        line = f'{user} ALL=(ALL) NOPASSWD: ALL'
        if d:
            sudoers = f'/etc/sudoers.d/{user}'
            target.write(sudoers, line)
        else:
            sudoers = '/etc/sudoers'
            target.deploy('upd_cfg',
                cfg = sudoers,
                src = f'^{user}',
                dst = line,
                report = 'quiet',
            )

        ok = 'wheel' in grp.get()
        ret, out, err = target.run(f'sudo -l -U {user}')
        ok = ok and '(ALL) NOPASSWD: ALL' in (line.strip() for line in out)

        return ok



from setux.core.module import Module


class Distro(Module):
    '''Set User as sudoer
    kw:
        usr : User name
        pub : Public key

    - Create User if not present
    - Add User to sudoers
    - Send User's public key
    '''

    def deploy(self, target, **kw):

        usr = kw['usr']
        pub = kw['pub']

        ok = target.user(usr)
        ok = ok and target.deploy('sudoers', user=usr)
        ok = ok and target.deploy('copy_id', **kw)

        return ok
